@@ -13,7 +13,58 @@
 namespace STGM
 {
 
-  //typedef enum { EMPTY=0 } IntersectionType;
+  /**
+    * @brief Digitizer
+    */
+
+   class CDigitizer
+   {
+    public:
+      CDigitizer(int *w, int nrow, int ncol, double delta) :
+        m_w(w), m_nrow(nrow), m_ncol(ncol), m_delta(delta),
+        x(STGM::CPoint2d(0,0)), y(STGM::CPoint2d(0,0))
+      {
+        /** safer: initialize */
+        for(int i=0;i<nrow*ncol;i++) m_w[i]=0;
+        m_nr = m_nrow-1;
+        m_nc = m_ncol-1;
+        m_d  = 0.5*m_delta-1e-6;
+      }
+
+      virtual ~CDigitizer() {};
+
+      template<typename T>
+      void start(T & obj)
+      {
+         //Rprintf("thread num: %d\n",omp_get_thread_num());
+
+         std::vector<STGM::CPoint2d> p = obj.getMinMaxPoints();
+         x=p[0]; y=p[1];
+
+         //Rprintf("[ %f %f ], [ %f %f ], \n",x[0],x[1],y[0],y[1]);
+
+         STGM::CBoundingRectangle &br = obj.getBoundingRectangle();
+         br.m_ymin=std::max(0,(int)((y[0]+m_d)/m_delta)); // y-coordinate is related to row number
+         br.m_xmin=std::max(0,(int)((x[0]+m_d)/m_delta)); // x-coordinate is related to col number
+         br.m_ymax=std::min(m_nr,(int)((y[1]-m_d)/m_delta));
+         br.m_xmax=std::min(m_nc,(int)((x[1]-m_d)/m_delta));
+
+         for(int i=br.m_ymin;i<(br.m_ymax+1);i++) {
+             for(int j=br.m_xmin;j<(br.m_xmax+1);j++) {
+                 /** change i and j for column/row major order */
+                 if(!m_w[i+j*m_nrow])
+                   if(obj.isInside((j+0.5)*m_delta,(i+0.5)*m_delta))
+                       m_w[i+j*m_nrow]=1;
+             }
+         }
+      }
+
+    private:
+      int *m_w, m_nr, m_nc, m_nrow, m_ncol;
+      double m_delta, m_d;
+      STGM::CPoint2d x,y;
+   };
+
   // Information about the intersection set
   enum IntersectionType  {
     EMPTY=0,          // 0
@@ -331,6 +382,7 @@ namespace STGM
        * @param delta
        */
       void digitizeCylinderIntersections(IntersectorCylinders &objects, int *w, int nPix, double delta);
+      void digitizeSpheroidIntersections(IntersectorSpheroids &objects, int *w, int nPix, double delta);
 
 } /* namespace STGM */
 
