@@ -20,7 +20,7 @@ extern "C" {
   SEXP SimulateSpheresAndIntersect(SEXP R_param, SEXP R_cond);
   SEXP GetSphereSystem(SEXP ext);
   SEXP SetupSphereSystem(SEXP R_vname, SEXP R_env, SEXP R_param, SEXP R_cond);
-  SEXP IntersectSphereSystem(SEXP ext, SEXP R_n, SEXP R_z);
+  SEXP IntersectSphereSystem(SEXP ext, SEXP R_n, SEXP R_z, SEXP R_intern);
   SEXP FinalizeSphereSystem(SEXP ext);
 
 #ifdef __cplusplus
@@ -44,7 +44,7 @@ public:
   STGM::Spheres &refObjects()  { return m_spheres; }
   const STGM::Spheres &refObjects() const { return m_spheres; }
 
-  void IntersectWithPlane(IntersectorSpheres &objects, STGM::CPlane &plane);
+  void IntersectWithPlane(IntersectorSpheres &objects, STGM::CPlane &plane, int intern);
 
   template<typename F> void simSpheres(F f, const char *label);
   void simSpheresPerfect(double mx, double sdx, const char *label);
